/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.diagram;

import java.awt.*;

/**
 * RectBackground - A rectangular node background.  With options, it can be
 * configured with round corners, and/or a shadow.  There are also color
 * options.
 *
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: RectBackground.java,v 1.4 1996/08/14 05:48:56 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/diagram/RectBackground.java,v $
 */

public class RectBackground extends NodeBackground {

    private RectColors normal = new RectColors( Color.white, Color.black );
    private RectColors arm = new RectColors( Color.gray, Color.black );
    private RectColors hilite = new RectColors( Color.yellow, Color.black );
    private RectColors mark = new RectColors( Color.cyan, Color.black );
    private Color shadow_color = Color.gray;
    private int border_width = 1;
    private int shadow_offset = 5;
    private int margin = 2;
    private boolean round_corners = false;
    private boolean threed_borders = true;
    private boolean transparent_background = false;
    private Color top_shadow = Color.white;
    private Color bottom_shadow = Color.black;
    private boolean armed = false;

	/**
	 * returns the left, right, top, and bottom insets of the background of the
	 * current style.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Insets insets() {
        int inset = border_width + margin;
        if ( shadow_offset < 0 ) {
            shadow_offset = 0;
        }
        if ( transparent_background ) {
            return new Insets( inset, inset,
                               inset, inset );
        }
        else {
            return new Insets( inset, inset,
                               inset + shadow_offset, inset + shadow_offset );
        }
    }

	/**
	 * draws the background, with border and shadow if present
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void draw( Graphics g, Point location, Dimension size,
                         boolean hilited, boolean disabled,
                         boolean armed, boolean marked ) {
        armed = false;

        int width = size.width;
        int height = size.height;

        if ( ( shadow_offset > 0 ) &&
             ( ! transparent_background ) ) {
            width -= shadow_offset;
            height -= shadow_offset;
        }

        RectColors colors = normal;

        if ( armed ) {
            colors = arm;
        }
        else if ( hilited ) {
            colors = hilite;
        }
        else if ( marked ) {
            colors = mark;
        }

        if ( ( shadow_offset > 0 ) &&
             ( ! transparent_background ) ) {
            g.setColor( shadow_color );
            if ( round_corners && ( ! threed_borders ) ) {
                g.fillRoundRect( location.x + shadow_offset,
                                 location.y + shadow_offset,
                                 width, height,
                                 ( margin * 8 ), ( margin * 8 ) );
            }
            else {
                g.fillRect( location.x + shadow_offset, location.y + shadow_offset,
                            width, height );
            }
        }

        g.setColor( colors.background );
        if ( ! transparent_background ) {
            if ( round_corners &&
                 ( ! threed_borders ) ) {
                g.fillRoundRect( location.x, location.y, width, height,
                                 ( margin * 8 ), ( margin * 8 ) );
            }
            else {
                g.fillRect( location.x, location.y, width, height );
            }
        }

        g.setColor( colors.border );

        for ( int idx = 0; idx < border_width; idx++ ) {
            if ( round_corners && ( ! threed_borders ) ) {
                g.drawRoundRect( location.x + idx, location.y + idx,
                            ( ( width - 1 ) - ( idx * 2 ) ),
                            ( ( height - 1 ) - ( idx * 2 ) ),
                            ( margin * 8 - 2 ), ( margin * 8 - 2 ) );
            }
            else {
                if ( threed_borders ) {
                    g.setColor( top_shadow );
                    g.drawLine( location.x + idx, location.y + idx,
                                location.x + idx,
                                location.y + ( ( height - 1 ) - ( idx ) ) );
                    g.drawLine( location.x + idx, location.y + idx,
                                location.x + ( ( width - 1 ) - ( idx ) ),
                                location.y + idx );
                    g.setColor( bottom_shadow );
                    g.drawLine( location.x + idx,
                                location.y + ( ( height - 1 ) - ( idx ) ),
                                location.x + ( ( width - 1 ) - ( idx ) ),
                                location.y + ( ( height - 1 ) - ( idx ) ) );
                    g.drawLine( location.x + ( ( width - 1 ) - ( idx ) ),
                                location.y + idx,
                                location.x + ( ( width - 1 ) - ( idx ) ),
                                location.y + ( ( height - 1 ) - ( idx ) ) );
                }
                else {
                    g.drawRect( location.x + idx, location.y + idx,
                                ( ( width - 1 ) - ( idx * 2 ) ),
                                ( ( height - 1 ) - ( idx * 2 ) ) );
                }
            }
        }
    }

	/**
	 * shows the background as armed or removes arm color, by drawing in XOR
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return a graphics object with which parts of the arm color can be
	 * erased
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Graphics arm( Graphics g, Point location, Dimension size, boolean hilited ) {

        armed = true;

        int width = size.width;
        int height = size.height;

        if ( ( shadow_offset > 0 ) &&
             ( ! transparent_background ) ) {
            width -= shadow_offset;
            height -= shadow_offset;
        }

        if ( threed_borders ) {
            for ( int idx = 0; idx < border_width; idx++ ) {
                g.setColor( bottom_shadow );
                g.setXORMode( top_shadow );
                g.drawLine( location.x + idx, location.y + idx,
                            location.x + idx,
                            location.y + ( ( height - 1 ) - ( idx ) ) );
                g.drawLine( location.x + idx + 1, location.y + idx,
                            location.x + ( ( width - 1 ) - ( idx ) ),
                            location.y + idx );
                g.setColor( bottom_shadow );
                g.setXORMode( top_shadow );
                g.drawLine( location.x + idx,
                            location.y + ( ( height - 1 ) - idx ),
                            location.x + ( ( width - 1 ) - ( idx ) ),
                            location.y + ( ( height - 1 ) - ( idx ) ) );
                g.drawLine( location.x + ( ( width - 1 ) - idx ),
                            location.y + idx,
                            location.x + ( ( width - 1 ) - ( idx ) ),
                            location.y + ( ( height - 1 ) - ( idx ) ) - 1 );
            }

            return null;
        }

        g.setColor( arm.background );
        if ( hilited ) {
            g.setXORMode( hilite.background );
        }
        else {
            g.setXORMode( normal.background );
        }

        if ( round_corners && ( ! threed_borders ) ) {
            g.fillRoundRect( location.x + border_width, location.y + border_width,
                             width - ( border_width * 2 ), height - ( border_width * 2 ),
                             ( margin * 8 ), ( margin * 8 ) );
        }
        else {
            g.fillRect( location.x + border_width, location.y + border_width,
                        width - ( border_width * 2 ), height - ( border_width * 2 ) );
        }

        return g;
    }


	/**
	 * shows the background as armed or removes arm color, by drawing in XOR
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return a graphics object with which parts of the arm color can be
	 * erased
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public Graphics disarm( Graphics g, Point location, Dimension size, boolean hilited ) {
        if ( armed ) {
            armed = false;
            return arm( g, location, size, hilited ); // redraw in XOR to erase
        }
        return null;
    }

	/**
	 * sets the color of the border
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setBorderColor( Color color ) {
        normal.border = color;
        arm.border = color;
        hilite.border = color;
    }

	/**
	 * sets the color of the normal background
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setBackground( Color color ) {
        normal.background = color;
    }

	/**
	 * sets the color of the background if armed
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setArmColor( Color color ) {
        arm.background = color;
    }

	/**
	 * sets the color of the background if hilited
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setHiliteColor( Color color ) {
        hilite.background = color;
    }

	/**
	 * sets the width of the background border, in pixels
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setBorderWidth( int width ) {
        border_width = width;
    }

	/**
	 * sets the number of pixels that the shadow, if any, will be offset, equally
	 * to the right and down from the rest of the background.  Zero to turn off
	 * the shadow.
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setShadowOffset( int offset ) {
        shadow_offset = offset;
    }

	/**
	 * Sets the color of the shadow, if any
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void setShadowColor( Color color ) {
        shadow_color = color;
    }

	/**
	 * sets a flag whether or not to round the corners on the rectangular background
	 * border (unless 3-D effect).
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void roundCorners( boolean round_them ) {
        round_corners = round_them;
    }

	/**
	 * sets a flag whether or not to draw 3-D border effect
	 * (implies no rounded corners)
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void threeDBorders( boolean threed_it ) {
        threed_borders = threed_it;
    }

	/**
	 * sets a flag whether or not make the background transparent
	 * (implies no shadow)
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void transparentBackground( boolean transparent_background ) {
        this.transparent_background = transparent_background;
    }
}

class RectColors {
    protected Color background;
    protected Color border;

    protected RectColors( Color background, Color border ) {
        this.background = background;
        this.border = border;
    }
}
