/**
 * Copyright(c) 1996 DTAI, Incorporated (http://www.dtai.com)
 *
 *                        All rights reserved
 *
 * Permission to use, copy, modify and distribute this material for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of DTAI, Incorporated not be used in
 * advertising or publicity pertaining to this material without the
 * specific, prior written permission of an authorized representative of
 * DTAI, Incorporated.
 *
 * DTAI, INCORPORATED MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES,
 * EXPRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST
 * INFRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL DTAI, INCORPORATED OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELATING
 * TO THE SOFTWARE.
 */

package COM.dtai.gui;

import java.awt.*;
import java.util.*;
import java.awt.image.*;

/**
 * ScrolledGadgetManager - implements a scrolled window for a GadgetManager,
 * only.  This scrolls a GadgetManager by telling the GadgetManager to draw
 * itself at different offsets from the origin, based on the scrollbars.
 *
 * @see		COM.dtai.gui.GadgetManager
 * @version	1.0b1
 * @author	DTAI, Incorporated
 *
 * $Id: ScrolledGadgetManager.java,v 1.3 1996/08/14 05:48:57 kadel Exp $
 *
 * $Source: /cvs/java/classes/COM/dtai/gui/ScrolledGadgetManager.java,v $
 */

public class ScrolledGadgetManager extends Panel implements ImageObserver {

    private Scrollbar vert;
    private Scrollbar horiz;
    private GadgetManager child;
    private boolean scrollbars_started = false;
    private boolean suppress_repaint = false;

    // if these are true and child is smaller than clipping area
    // in that direction, it will be centered in that direction
    private boolean center_horiz = false;
    private boolean center_vert = false;
    private boolean old_scrollbars = false;

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public ScrolledGadgetManager() {

        setLayout( new BorderLayout() );

        horiz = new Scrollbar( Scrollbar.HORIZONTAL );
        vert = new Scrollbar( Scrollbar.VERTICAL );

        add( "South", horiz );
        add( "East", vert );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void centerHorizontally( boolean center ) {
        center_horiz = center;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void centerVertically( boolean center ) {
        center_vert = center;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception RuntimeException thrown if the child has already been added
	 * @see CLASSNAME#METHOD
	 */
    public void addChild( GadgetManager child ) throws RuntimeException {
        if ( this.child != null ) {
            throw new RuntimeException();
        }
        this.child = child;
        add( "Center", child );
        child.addImageObserver( this );
    }

    protected synchronized void moveChild () {
        Dimension clip_dim = child.size ();
        Dimension child_dim = child.getImageSize ();

        int new_x = 0;
        if ( horiz.isVisible() ) {
            new_x = -horiz.getValue ();
        }
        else if ( center_horiz ) {
            new_x = Math.max( 0, ( ( clip_dim.width - child_dim.width ) / 2 ) );
        }

        int new_y = 0;
        if ( vert.isVisible() ) {
            new_y = -vert.getValue ();
        }
        else if ( center_vert ) {
            new_y = Math.max( 0, ( ( clip_dim.height - child_dim.height ) / 2 ) );
        }

        child.setOffsets( new_x, new_y );
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean handleEvent ( Event evt ) {
        if ( child == null ) {
            return false;
        }
        Dimension clip_dim = child.size ();
        if (evt.target == vert) {
            moveChild ();
            return true;
        }
        else if (evt.target == horiz) {
            moveChild ();
            return true;
        }
        else if ( ( evt.target == this ) &&
                  ( evt.id == Event.MOUSE_MOVE ) ) {
            if ( ! scrollbars_started ) {
                suppress_repaint = true;
                reset();
                scrollbars_started = true;
            }
            return false;
        }
        else {
            return false;
        }
    }

    private void reset() {
        Dimension clip_dim = child.size ();
        Dimension child_dim = child.getImageSize ();
        int vert_diff = Math.max( 0, child_dim.height - clip_dim.height );
        if ( old_scrollbars ) {
            vert.setValues ( Math.min( vert_diff, vert.getValue () ), clip_dim.height,
                             0, child_dim.height );
        }
        else {
            vert.setValues ( Math.min( vert_diff, vert.getValue () ), clip_dim.height,
                             0, vert_diff );
        }
        vert.setPageIncrement (
                         Math.max (1, Math.min (child_dim.height, clip_dim.height)));
        vert.setLineIncrement (Math.max (1, (clip_dim.height / 10)));

        int horiz_diff = Math.max( 0, child_dim.width - clip_dim.width );

        if ( old_scrollbars ) {
            horiz.setValues ( Math.min( horiz_diff, horiz.getValue () ), clip_dim.width,
                              0, child_dim.width);
        }
        else {
            horiz.setValues ( Math.min( horiz_diff, horiz.getValue () ), clip_dim.width,
                              0, horiz_diff);
        }

        horiz.setPageIncrement (
                         Math.max (1, Math.min (child_dim.width, clip_dim.width)));
        horiz.setLineIncrement (Math.max (1, clip_dim.width / 10));

        boolean dorepaint = false;

        if ( clip_dim.width >= child_dim.width ) {
            if ( horiz.isVisible() ) {
                horiz.hide();
                dorepaint = true;
            }
        }
        else {
            if ( ! horiz.isVisible() ) {
                horiz.show();
                dorepaint = true;
            }
        }

        if ( clip_dim.height >= child_dim.height ) {
            if ( vert.isVisible() ) {
                vert.hide();
                dorepaint = true;
            }
        }
        else {
            if ( ! vert.isVisible() ) {
                vert.show();
                dorepaint = true;
            }
        }

        moveChild ();

        if ( dorepaint ) {
            layout();
        }
        if ( ! suppress_repaint ) {
            child.repaint();
            suppress_repaint = false;
        }
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public boolean imageUpdate( Image image, int infoflags,
                                int x, int y, int width, int height ) {
        reset();
        return true;
    }

	/**
	 * DESCRIPTION
	 *
	 * @param PARAMNAME DESCRIPTION
	 * @return DESCRIPTION
	 * @exception CLASSNAME DESCRIPTION
	 * @see CLASSNAME#METHOD
	 */
    public void paint( Graphics g ) {
        reset();
    }
}
